@@ -2,7 +2,6 @@
 
 #include "R.h"
 #include "Rinternals.h"
-#define printf Rprintf
 
 #ifndef _MSC_VER // microsoft
 #ifndef bool
@@ -14,12 +13,12 @@
 
 #define Dirs_(iTerm,iPred) Dirs[(iTerm) + (iPred)*(nMaxTerms)]
 
-static SEXP AllowedFunc;
-static SEXP AllowedEnv;
-static int  nArgs;
-static bool First;
+static SEXP AllowedFuncGlobal;
+static SEXP AllowedEnvGlobal;
+static int  nArgsGlobal;
+static bool FirstGlobal;
 
-// Initialize the R function AllowedFunc from the Allowed function
+// Initialize the R function AllowedFuncGlobal from the Allowed function
 // argument which was passed into ForwardPassR.
 // For efficiency, we initialize once here rather than in IsAllowed.
 //
@@ -46,37 +45,37 @@ static bool First;
 
 void InitAllowedFunc(
         SEXP Allowed, // can be NULL
-        int nAllowedFuncArgs, SEXP Env,
-        const char* sPredNames[], int nPreds)
+        int nAllowedArgs, SEXP Env,
+        const char** sPredNames, int nPreds)
 {
-    if(Allowed == NULL)
-        AllowedFunc = NULL;
+    if(Allowed == R_NilValue)
+        AllowedFuncGlobal = NULL;
     else {
-        if(nAllowedFuncArgs < 3 || nAllowedFuncArgs > 5)
-            error("Bad nAllowedFuncArgs %d", nAllowedFuncArgs);
+        if(nAllowedArgs < 3 || nAllowedArgs > 5)
+            error("Bad nAllowedArgs %d", nAllowedArgs);
 
-        AllowedEnv = Env;
-        nArgs = nAllowedFuncArgs;
+        AllowedEnvGlobal = Env;
+        nArgsGlobal = nAllowedArgs;
 
         // the UNPROTECT for the PROTECT below is in FreeAllowedFunc()
-        PROTECT(AllowedFunc = allocList(1 + nAllowedFuncArgs));
+        PROTECT(AllowedFuncGlobal = allocList(1 + nAllowedArgs));
 
-        SEXP s = AllowedFunc;   // 1st element is the function
+        SEXP s = AllowedFuncGlobal; // 1st element is the function
         SETCAR(s, Allowed);
         SET_TYPEOF(s, LANGSXP);
 
-        s = CDR(s);             // 2nd element is "degree"
+        s = CDR(s);                 // 2nd element is "degree"
         SETCAR(s, allocVector(INTSXP, 1));
 
-        s = CDR(s);             // 3rd element is "pred"
+        s = CDR(s);                 // 3rd element is "pred"
         SETCAR(s, allocVector(INTSXP, 1));
 
-        s = CDR(s);             // 4th element is "parents"
+        s = CDR(s);                 // 4th element is "parents"
         SETCAR(s, allocVector(INTSXP, nPreds));
 
-        if(nAllowedFuncArgs >= 4) {
+        if(nAllowedArgs >= 4) {
             SEXP namesx;
-            s = CDR(s);        // 5th element is "namesx"
+            s = CDR(s);             // 5th element is "namesx"
             SETCAR(s, namesx = allocVector(STRSXP, nPreds));
             PROTECT(namesx);
             if(sPredNames == NULL)
@@ -85,28 +84,28 @@ void InitAllowedFunc(
                 SET_STRING_ELT(namesx, i, mkChar(sPredNames[i]));
             UNPROTECT(1);
         }
-        if(nAllowedFuncArgs >= 5) {
-            s = CDR(s);        // 6th element is "first"
+        if(nAllowedArgs >= 5) {
+            s = CDR(s);             // 6th element is "first"
             SETCAR(s, allocVector(LGLSXP, 1));
         }
     }
-    First = true;
+    FirstGlobal = true;
 }
 
 void FreeAllowedFunc(void)
 {
-    if(AllowedFunc != NULL) {
-        UNPROTECT(1);          // matches PROTECT in InitAllowedFunc
-        AllowedFunc = NULL;
+    if(AllowedFuncGlobal != NULL) {
+        UNPROTECT(1);           // matches PROTECT in InitAllowedFunc
+        AllowedFuncGlobal = NULL;
     }
 }
 
 static bool EvalAllowedFunc(void)
 {
-    if(AllowedFunc == NULL)
-        error("EvalAllowedFunc: AllowedFunc == NULL");
+    if(AllowedFuncGlobal == NULL)
+        error("EvalAllowedFunc: AllowedFuncGlobal == NULL");
 
-    SEXP s = eval(AllowedFunc, AllowedEnv);
+    SEXP s = eval(AllowedFuncGlobal, AllowedEnvGlobal);
 
     bool allowed;
     switch(TYPEOF(s)) {         // be fairly permissive with return type
@@ -136,7 +135,7 @@ static bool EvalAllowedFunc(void)
 //
 // This calls the R function Allowed which was passed in as a parameter to
 // ForwardPassR.  The fields of Allowed have been preallocated into
-// AllowedFunc and so all we do here is fill in the values and call eval.
+// AllowedFuncGlobal and so all we do here is fill in the values and call eval.
 
 bool IsAllowed(
     const int iPred,        // in: candidate predictor
@@ -145,10 +144,10 @@ bool IsAllowed(
     const int nPreds,       // in:
     const int nMaxTerms)    // in:
 {
-    if(AllowedFunc == NULL)
+    if(AllowedFuncGlobal == NULL)
        return TRUE;
 
-    SEXP s = AllowedFunc;           // 1st element is the function
+    SEXP s = AllowedFuncGlobal;     // 1st element is the function
     s = CDR(s);                     // 2nd element is "degree"
     INTEGER(CADR(s))[0] = iPred+1;  // 3rd element is "pred"
     int* p = INTEGER(CADDR(s));     // 4th element is "parents"
@@ -162,9 +161,9 @@ bool IsAllowed(
 
     // optional 5th element already initialized to predictor names
 
-    if(nArgs >= 5)                  // optional 6th element is "first"
-        *(LOGICAL(CAD4R(s))) = First;
-    First = false;
+    if(nArgsGlobal >= 5)            // optional 6th element is "first"
+        *(LOGICAL(CAD4R(s))) = FirstGlobal;
+    FirstGlobal = false;
 
     return EvalAllowedFunc();
 }
