@@ -12,3 +12,9 @@ allowed.func <- function(degree, pred, parents, namesx)
 }
 earth.mod2 <- earth(Volume~., data=trees, allowed=allowed.func)
 print(summary(earth.mod2))
+# multiple response model using class "Formula" and a factor predictor
+data(iris)
+earth.mod3 <- earth(Sepal.Length + Sepal.Width ~ Species, data=iris)
+print(summary(earth.mod3))
+plot(earth.mod3, nresponse="Sepal.Length", which=c(1,3), do.par=2, legend.pos="topleft")
+plotmo(earth.mod3, nresponse=1, pt.col=2, do.par=0)
